# Hands-on 3: Verification Workflow with SUNSET

## Goal
Create a SUNSET recipe and use the functions in the suite to reproduce the verification workflow from the previous hands-on exercises.

## 0. Copy the recipe template and modify it

```shell
# Go to the SUNSET directory
cd $SUNSET_DIR
# Create a copy of the recipe in your directory, adding your name in <user_name>
cp use_cases/PATC_2023/recipe_PATC_2023.yml use_cases/PATC_2023/recipe_PATC_2023-<user_name>.yml
# Open the recipe with a text editor such as vim
vim use_cases/PATC_2023/recipe_PATC_2023-<user_name>.yml
```

Once you have opened your recipe, it's time to edit the contents. For this example, we will evaluate temperature-at-surface (tas) using MeteoFrance System 7 data as our experiment and ERA5 as our reference dataset, for the initialization month of November. 

```yaml
Description:
 Author: <___>
 Description:  Calibration and skill assessment of MeteoFrance System 7 surface temperature
Analysis:
 Horizon: Seasonal
 Variables:
   name: tas
   freq: monthly_mean
   units:  <___> # Choose your units: C or K
 Datasets:
   System:
     name: Meteo-France-System7
     Multimodel: no
   Reference:
     name:  ERA5
 Time:
   sdate: '1101'
   fcst_year: '2020'
   hcst_start: '1993'
   hcst_end: '2016'
   ftime_min: <___> # Choose the first time step! A number from 1 to 6
   ftime_max: <___> # Choose the last time step! A number from 1 to 6
 Region:
   name:  "EU"
   latmin:  20
   latmax:  80
   lonmin:  -20
   lonmax:  40
 Regrid:
   method:  bilinear
   type:  'r360x181' # options: to_system, to_reference, self-defined grid
 Workflow:
   Anomalies:
     compute: yes
     cross_validation: yes
     save: 'none'
   Calibration:
     method: evmos
     save: 'none'
   Skill:
     metric: RPSS,  BSS10,  BSS90
     cross_validation: yes
     save: 'all'
   Probabilities:
     percentiles:  [[1/3,  2/3],  [1/10,  9/10]]
     save: 'none'
   Visualization:
     plots: skill_metrics,  forecast_ensemble_mean,  most_likely_terciles
     multi_panel: no
     projection: cylindrical_equidistant
     mask_terciles:  no  # CHECK
     dots: no  # CHECK 
   ncores: 10
   remove_NAs: yes
   Output_format: S2S4E
Run:
 Loglevel: ____ # DEBUG or INFO
 Terminal: yes
 output_dir: /gpfs/scratch/bsc32/
```

## 1. Load the required SUNSET modules and read the recipe
```r
# Load modules
source("modules/Loading/Loading.R")
source("modules/Units/Units.R")
source("modules/Calibration/Calibration.R")
source("modules/Anomalies/Anomalies.R")
source("modules/Skill/Skill.R")
source("modules/Saving/Saving.R")
source("modules/Visualization/Visualization.R")

# Read recipe
recipe_file <- "use_cases/PATC_2023/recipe_PATC_2023-<______>.yml"
recipe <- prepare_outputs(recipe_file)
```
**Questions**
Read the logs!
(1) Did your recipe pass the check? Did you get any warnings?
(2) Where will your outputs be saved? Copy and paste this directory somewhere, so that you can check it later!

## 2. Load the data and change the units

```r
# Load datasets
data <- Loading(recipe)
# Change units
data <- Units(recipe, data)
```

**Questions**

(1) What is the structure of `data`? Tip: you can use functions like `class()`, `names()` or `str()` to gain information about the structure of the object and its contents.
```r
class(data)
names(data)
class(data$____)
str(data, max.level = 2)
```

(2) Pay attention to the log messages: Did your units get converted? Are the new units what you expect? You can check the metadata of any of the objects in data. SUNSET also provides the `data_summary()` function, which lets you have a quick look at your objects:

```r
# Check the new units and data of hcst, fcst and/or obs. Are they all the same?
data$____$attrs$Variable$metadata$tas$units
data_summary(data$____, recipe)
```
(3) What are the dimensions of the datasets? Are they consistent with what is requested in the recipe? Tip: Check the data summary!

## 3. Calibrate the data and compute the anomalies
SUNSET has a few module to perform post-processing on the experimental and the reference datasets. The Calibration module performs the bias correction method indicated in the recipe, using the `CSTools::CST_Calibration()` function. The Anomalies module removes the climatology using `CSTools::CST_Anomaly()`

```r
# Calibrate the data
data <- Calibration(recipe, data)
# Compute anomalies
data <- Anomalies(recipe, data)
```

**Questions**

(1) Verify that you now have anomaly values instead of the original full field. Tip: Use `data_summary()` like in the previous example and pay attention to the new values.

## 4. Evaluate the model skill and compute the probability thresholds
The Skill module returns a list of all the evaluation metrics requested in the recipe, in the shape of multi-dimensional arrays. In this case, we will compute three metrics:

- **RPSS (Ranked Probability Skill Score)**: This skill score measures how well a forecast predicts the probability of the tercile categories (below normal, normal and above-normal), compared to the climatology.
- **BSS10 and BSS90 (Brier Skill Score):** This skill score measures how well a forecast predicts the probability of the 10th percentile and 90th percentile extreme events, compared to the climatology.

```r
# Compute skill metrics
skill_metrics <- Skill(recipe, data)
# Compute percentiles and probability bins
probabilities <- Probabilities(recipe, data)
```
**Questions**

(1) What is the structure of `skill_metrics`? Which metrics were computed? What dimensions do they have? Tip: use `str()` and `dim()`.

(2) What is the structure of `probabilities`? Can you list the probability categories and the percentiles? Tip: use `str()`.

## 5. Plotting the results

The Visualization module will generate the three types of maps we requested in the recipe: 
- Skill maps to visualize the skill distribution of the model, for each metric.
- The ensemble mean of the calibrated forecast anomalies.
- A map showing the most likely tercile category for each point in the grid.
```r
# Plot data
Visualization(recipe, data, skill_metrics, probabilities, significance = T)
```

Now, you can `cd` to the the output directory and inspect the contents of the `plots/` subdirectory. The plots are png files that can be visualized with the `display` command. They have a descriptive name including the content of the plot, the date and the forecast time.

**Questions**

(1) Let's take a look at the forecast ensemble mean. What is the sign of the anomalies over Spain? In what regions are the anomalous temperatures strongest?

(2) Let's take a look at the skill metrics RPSS, BSS10 and BSS90. In what regions is the forecast most skillful? Tip: Positive values indicate that the model is a better predictor than the climatology, with 1 being the perfect score.

(3) Let's take a look at the Most Likely Terciles plots. This plot indicates the probability of the temperature being below normal, near-normal or above normal. What is the most likely category for Spain?
